use std::{fmt::Display, num::NonZeroU8};

use termion::color::{self, Color};

#[derive(PartialEq, Eq, Clone, Copy)]
pub enum Player {
	Human,
	Computer,
}

impl Player {
	pub fn invert(&mut self) {
		match self {
			Self::Human => *self = Self::Computer,
			Self::Computer => *self = Self::Human,
		}
	}

	pub fn color(&self) -> &dyn Color {
		match self {
			Player::Computer => &color::LightBlack,
			Player::Human => &color::Magenta,
		}
	}
}

impl Display for Player {
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		let s = match self {
			Self::Computer => "Max",
			Self::Human => "Min",
		};
		write!(f, "{s}")
	}
}

#[derive(PartialEq, Eq, Clone, Copy)]
pub enum Symb {
	Number(NonZeroU8),
	Zero,
	Plus,
	Minus,
	Times,
	Div,
}

impl Display for Symb {
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		match self {
			Self::Number(x) => x.fmt(f),
			Self::Zero => '0'.fmt(f),
			Self::Plus => '+'.fmt(f),
			Self::Minus => '-'.fmt(f),
			Self::Div => '÷'.fmt(f),
			Self::Times => '×'.fmt(f),
		}
	}
}

impl Symb {
	/// Is this symbol a plain binary operator?
	pub fn is_binop(&self) -> bool {
		match self {
			Symb::Div | Symb::Plus | Symb::Times => true,
			_ => false,
		}
	}

	pub fn is_minus(&self) -> bool {
		self == &Self::Minus
	}
}
